# encoding: utf-8
from base import BaseRest
from objects import OrganizationMembership


class OrganizationMemberships(BaseRest):
    '''
        Customers (end-users) can be segmented into organization_memberships
        url: https://developer.zendesk.com/rest_api/docs/core/organization_memberships
    '''

    def __init__(self, base):
        super(OrganizationMemberships, self).__init__(
            base, 'organization_memberships', OrganizationMembership)

    @classmethod
    def save_bulk(self, organization_memberships):
        pass



# encoding: utf-8
from exceptions import NotImplementedError
from ..custom_fields import CustomFields
from inflection import singularize


class Base(object):
    def __init__(self, resource, **kwargs):
        self.save = self.__save
        self.resource = resource
        self.__dict__ = kwargs
        singular_resource = singularize(resource)
        name_custom_fields = '{}_fields'.format(singular_resource)
        self.CustomFields = CustomFields(**kwargs.pop(name_custom_fields, {}))
        map(lambda x: setattr(self, x[0], x[1]), kwargs.items())

    @classmethod
    def resource(self):
        raise NotImplementedError("Method not implemented!")

    @classmethod
    def save(cls, **kwargs):
        raise NotImplementedError("Method not implemented!")

    def __save(self):
        raise NotImplementedError("Method not implemented!")

    @classmethod
    def create(cls, zendesk, **kwargs):
        resource = "{}.json".format(cls.resource)
        singular_resource = singularize(cls.resource)
        data = {singular_resource: kwargs}
        resp = zendesk._request(resource, 'POST', **data)
        if resp.status_code != 201:
            from exceptions import RequestException
            raise RequestException(resp.status_code)
        return cls(**resp.json().get(singular_resource))

    @classmethod
    def create_many(cls, zendesk):
        raise NotImplementedError("Method not implemented!")

    @property
    def __unicode__(self):
        name = "{} - {}".format(
            getattr(self, "name", ""), getattr(self, "id", ""))
        if not name:
            return super(Base, self).__unicode__()
        return name



# encoding: utf-8
from base import BaseRest
from objects import User


class Users(BaseRest):
    def __init__(self, base):
        super(Users, self).__init__(base, 'users', User)

    def get_per_group(self, id_group, **kwargs):
        resource = "groups/{}/{}".format(id_group, self.resource)
        return super(Users, self).get(resource, **kwargs)

    def get_per_organization(self, id_organization, **kwargs):
        resource = "organizations/{}/{}".format(id_organization, self.resource)
        return super(Users, self).get(resource, **kwargs)



# encoding: utf-8


class RequestException(Exception):
    def __init__(self, status_code=500, content=None):
        message = "Status: {}. Problem with the request exiting.".format(
            status_code)
        self.status_code = status_code
        self.content = content
        super(RequestException, self).__init__(message)


class BulkExceededLimit(Exception):
    def __init__(self):
        message = "The bulk can not exceed 100 objects"
        super(BulkExceededLimit, self).__init__(message)



# encoding: utf-8
from base import Base
from ..custom_fields import CustomFields


class User(Base):
    def __init__(self, **kwargs):
        super(User, self).__init__('users', **kwargs)

    @classmethod
    def save(cls, **kwargs):
        print("This is a class method.")

    def __save(self):
        print("This is an instance method.")

    @classmethod
    def resource(cls):
        return "users"


# encoding: utf-8
from base import Base


class Organization(Base):
    resource = "organizations"

    __str__ = lambda x: "{} - {}".format(x.id, x.name)

    def __init__(self, **kwargs):
        super(Organization, self).__init__('organizations', **kwargs)

    def create_ticket(self):
        pass

    def get_tickets(self):
        pass

    def get_users(self):
        pass

    @classmethod
    def save(cls, **kwargs):
        print("This is a class method.")

    def __save(self):
        print("This is an instance method.")



# encoding: utf-8
from base import BaseRest
from objects import Organization
from helper import separete_into_groups


class Organizations(BaseRest):
    '''
        Customers (end-users) can be segmented into organizations
        url: https://developer.zendesk.com/rest_api/docs/core/organizations
    '''

    def __init__(self, base):
        super(Organizations, self).__init__(base, 'organizations', Organization)

    @classmethod
    def save_bulk(self, organizations):
        pass

    @classmethod
    def create_bulk(self, zendesk, organizations):
    	url = "organizations/create_many.json"
    	groups = separete_into_groups(organizations, 100)
        has_pendent_groups = True
        while has_pendent_groups:
            errors = []
            for orgs in groups:
                try:
                    data = {"organizations": orgs}
                    resp = zendesk.Organizations.base._request(url, 'PUT', **data)
                except Exception:
                    errors.append(orgs)
            if errors:
                groups = map(lambda x: x, errors)
            else:
                has_pendent_groups = False
        return jobs



# encoding: utf-8


class CustomFields(object):
    def __init__(self, **kwargs):
        map(lambda x: setattr(self, x[0], x[1]), kwargs.items())

    def __getattribute__(self, attr):
        try:
            return super(CustomFields, self).__getattribute__(attr)
        except AttributeError:
            return None



# encoding: utf-8
from base import BaseRest
from objects import Ticket


class Tickets(BaseRest):
    '''
        TODO
    '''
    def __init__(self, base):
        super(Tickets, self).__init__(base, 'tickets', Ticket)



# encoding: utf-8
from base import Base


class OrganizationMembership(Base):
    resource = "organization_memberships"

    __str__ = lambda x: x.id

    def __init__(self, **kwargs):
        super(OrganizationMembership, self).__init__(
            'organization_memberships', **kwargs)

    @classmethod
    def save(cls, **kwargs):
        print("This is a class method.")

    def __save(self):
        print("This is an instance method.")



# encoding: utf-8


def separete_into_groups(list_objs, size_group=100):
    groups = []
    group = []
    for org in list_objs:
        group.append(org)
        if len(group) >= size_group:
            groups.append(group)
            group = []
    if group:
        groups.append(group)
    return groups



# encoding: utf-8
import requests
import json
from inflection import singularize
from helper import separete_into_groups
import exceptions
from custom_exceptions import BulkExceededLimit, RequestException


class BaseZenDesk(object):
    def __init__(self, hostname, user, password, timeout=15):
        self.host = "https://{}.zendesk.com/api/v2/".format(hostname)
        self.auth = (user, password)
        self.timeout = timeout

    def _request(self, resource, method='get', **kwargs):
        '''
            TODO
        '''
        _method = getattr(requests, method.lower())
        headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
        url = "{host}{resource}".format(host=self.host, resource=resource)
        return _method(url, auth=self.auth, data=json.dumps(kwargs),
                       timeout=self.timeout, headers=headers)


class BaseRest(object):
    def __init__(self, base, resource, class_object):
        self.base = base
        self.resource = resource
        self.class_object = class_object

    def get(self, resource=None, page=1, per_page=10, **kwargs):
        resource = resource or self.resource
        endpoint = "{}.json?page={}&per_page={}".format(
            resource, page, per_page)
        resp = self.base._request(endpoint, **kwargs)
        if resp.status_code != 200:
            content = resp.json() if getattr(resp, 'json') else {}
            raise RequestException(resp.status_code, content=content)
        resp = resp.json()
        # return resp
        items = resp.pop(self.resource)
        resp.update(items=map(lambda x: self.class_object(**x), items))
        return resp

    def get_one(self, id_object, resource=None):
        resource = resource or self.resource
        url = "{}/{}.json".format(resource, id_object)
        resp = self.base._request(url)
        if resp.status_code != 200:
            content = resp.json() if getattr(resp, 'json') else {}
            raise RequestException(resp.status_code, content=content)
        return self.class_object(**resp.json())

    def show_many(self, resource=None, name_field='ids', fields=[]):
        if len(fields) > 100:
            raise Exception('identificares the limit is 100!')
        resource = resource or self.resource
        url = "{}/show_many.json?{}={}".format(
            resource, name_field, fields.split(','))
        resp = self.base._request(url)
        if resp.status_code != 200:
            content = resp.json() if getattr(resp, 'json') else {}
            raise RequestException(resp.status_code, content=content)
        resp = resp.json()
        items = resp.pop(self.resource)
        resp.update(items=map(lambda x: self.class_object(**x), items))
        return resp

    def create(self, resource=None, **kwargs):
        resource = resource or self.resource
        url = "{}.json".format(resource)
        singular_resource = singularize(resource)
        data = {
            singular_resource: kwargs,
        }
        resp = self.base._request(url, 'POST', **data)
        if resp.status_code != 201:
            content = resp.json() if getattr(resp, 'json') else {}
            raise RequestException(resp.status_code, content=content)
        return self.class_object(**resp.json().get(singularize(resource)))

    def create_many(self, list_objects, resource=None):
        jobs = []
        resource = resource or self.resource
        url = "{}/create_many.json".format(resource)
        groups = separete_into_groups(list_objects, 100)
        has_pendent_groups = True
        while has_pendent_groups:
            errors = []
            for group in groups:
                try:
                    data = {resource: group}
                    resp = self.base._request(url, 'POST', **data)
                    jobs.append(resp.json())
                except Exception:
                    errors.append(group)
            if errors:
                groups = map(lambda x: x, errors)
            else:
                has_pendent_groups = False
        return jobs

    def put(self, id_object, resource=None, **kwargs):
        resource = resource or self.resource
        url = "{}/{}.json".format(resource, id_object)
        data = {singularize(resource): kwargs}
        resp = self.base._request(url, 'PUT', **data)
        if resp.status_code != 200:
            content = resp.json() if getattr(resp, 'json') else {}
            raise RequestException(resp.status_code, content=content)
        return self.class_object(**resp.json())

    def bulk_put_many(self, documents, resource=None, limit=100):
        if limit > 100:
            raise BulkExceededLimit
        resource = resource or self.resource
        url = "{}/update_many.json".format(resource)
        groups = separete_into_groups(documents, limit)
        has_pendent_groups = True
        jobs = []
        while has_pendent_groups:
            errors = []
            for group in groups:
                try:
                    data = {resource: group}
                    resp = self.base._request(url, 'PUT', **data)
                    jobs.append(resp.json())
                except Exception:
                    errors.append(group)
            if errors:
                groups = map(lambda x: x, errors)
            else:
                has_pendent_groups = False
        return jobs

    def put_many(self):
        raise exceptions.NotImplemented("Method not implemented!")

    def delete(self, id_object, resource=None):
        resource = resource or self.resource
        url = "{}/{}.json".format(resource, id_object)
        resp = self.base._request(url, 'DELETE')
        if resp.status_code != 200:
            content = resp.json() if getattr(resp, 'json') else {}
            raise RequestException(resp.status_code, content=content)

    def delete_many(self, list_ids, resource=None, name_field='ids', limit=100):
        resource = resource or self.resource
        groups = separete_into_groups(list_ids, limit)
        has_pendent_groups = True
        responses = []
        while has_pendent_groups:
            errors = []
            for group in groups:
                try:
                    url = "{}/destroy_many.json?{}={}.json".format(
                        resource, name_field, ','.join(group))
                    resp = self.base._request(url, 'DELETE')
                    responses.append(resp.json())
                except Exception:
                    errors.append(group)
            if errors:
                groups = map(lambda x: x, errors)
            else:
                has_pendent_groups = False
        return responses



'''
zendesk: Communicator with the Zendesk API

Note that "python setup.py test" invokes pytest on the package. With appropriately
configured setup.cfg, this will check both xxx_test modules and docstrings.

Copyright 2015, Rafael Portugal.
Licensed under MIT.
'''
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

# This is a plug-in for setuptools that will invoke py.test
# when you run python setup.py test
class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest  # import here, because outside the required eggs aren't loaded yet
        sys.exit(pytest.main(self.test_args))


version = "0.2"

setup(name="zendesk_integration",
      version=version,
      description="Communicator with the Zendesk API",
      long_description=open("README.rst").read(),
      classifiers=[ # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 1 - Planning',
        'Programming Language :: Python'
      ],
      keywords="zendesk api", # Separate with spaces
      author="Rafael Portugal",
      author_email="rafaelportugal05@gmail.com",
      url="",
      license="MIT",
      packages=find_packages(exclude=['examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      tests_require=['pytest'],
      cmdclass={'test': PyTest},

      # TODO: List of packages that this one depends upon:
      install_requires=['requests==2.5.1', 'inflection==0.3.1', 'mock'],
      # TODO: List executable scripts, provided by the package (this is just an example)
      entry_points={
        'console_scripts':
            ['zendesk=zendesk:main']
      }
)



'''
zendesk: Main module

Copyright 2015, Rafael Portugal
Licensed under MIT.
'''
# from .zendesk import *

from custom_exceptions import *
from base import BaseZenDesk
from tickets import Tickets
from users import Users
from organizations import Organizations
from organization_memberships import OrganizationMemberships


class Zendesk(BaseZenDesk):
    def __init__(self, hostname, user, password, timeout=15):
        super(Zendesk, self).__init__(hostname, user, password, timeout=timeout)
        self.Tickets = Tickets(self)
        self.Users = Users(self)
        self.Organizations = Organizations(self)
        self.OrganizationMemberships = OrganizationMemberships(self)
